@@ -10,11 +10,11 @@ class GlobalApiTest(CoreTest):
     def setUp(self):
         super().setUp()
 
-        self.smast2 = self.meta.super_master(h=self.smast._h)
+        self.smast2 = self.meta.build_super_master(h=self.smast._h)
         self.smast.sentinel_register(name="test", code=jtc.basic)
 
-        self.mast = self.meta.master()
-        self.mast2 = self.meta.super_master(h=self.mast._h)
+        self.mast = self.meta.build_master()
+        self.mast2 = self.meta.build_super_master(h=self.mast._h)
         self.mast.sentinel_register(name="test", code=jtc.basic)
 
     def tearDown(self):
